/*  Jxspectrum - a spectrum analyzer.
    Copyright (C) 1997  Jani Frilander

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.*/

#include "gui.h"
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qapp.h>
#include <stdio.h>
#include <fstream.h>

void SetSF(int speed);
void StartRecording   ();
void StopRecording    ();
void Quit();
void SetWinKaiser();
void SetWinBlackman();
void SetWinRectangle();
void SetOpmode(int mode);

Gui::~Gui()
{
    delete file;
    delete rate;
    delete delay;
    delete extraf;
    delete label;
    delete plot;
    delete menu;
}

Gui::Gui( QWidget *parent, const char *name )
    : QWidget( parent, name )
{
    
    file = new QPopupMenu();
    CHECK_PTR( file );
    file->insertItem( "Export",  this, SLOT(menu_export()));
    file->insertSeparator();
    file->insertItem( "Exit",  this, SLOT(menu_quit()));

    rate = new QPopupMenu();
    CHECK_PTR( rate );
    rate->insertItem( "40000", this, SLOT(menu_40000()) );
    rate->insertItem( "20000", this, SLOT(menu_20000()) );
    rate->insertItem( "10000", this, SLOT(menu_10000()) );
    
    delay = new QPopupMenu;
    CHECK_PTR( delay );
    delay->insertItem( "No delay", this, SLOT(menu_nodelay()));
    delay->insertItem( "10 ms", this, SLOT(menu_delay10()));
    delay->insertItem( "20 ms", this, SLOT(menu_delay20()));
    delay->insertItem( "40 ms", this, SLOT(menu_delay40()));
    delay->insertItem( "80 ms", this, SLOT(menu_delay80()));
    delay->insertItem( "160 ms", this, SLOT(menu_delay160()));
    delay->insertItem( "320 ms", this, SLOT(menu_delay320()));
    delay->insertSeparator();
    delay->insertItem( "Freeze", this, SLOT(menu_freeze()));
    
    extraf = new QPopupMenu;
    CHECK_PTR( extraf );
    extraf->insertItem( "Peak Display", this, SLOT(menu_peak()));
    extraf->insertItem( "Reset Values", this, SLOT(menu_reset()));
    extraf->insertItem( "Normal", this, SLOT(menu_normal()));
    
    windowf = new QPopupMenu;
    CHECK_PTR( windowf );
    windowf->insertItem("Kaiser", this, SLOT(menu_kaiser()));
    windowf->insertItem("Blackman", this, SLOT(menu_blackman()));
    windowf->insertItem("Rectangle", this, SLOT(menu_rectangle()));
    
    menu = new QMenuBar( this );
    CHECK_PTR( menu );
    menu->insertItem( "File", file );
    menu->insertItem( "Sample Rate", rate );
    menu->insertItem( "Delay", delay );
    menu->insertItem( "Window Function", windowf );
    menu->insertItem( "Extra Functions", extraf );
    
    label = new QLabel( this );
    CHECK_PTR( label );
    label->setMinimumSize(100,20);
    label->setGeometry( 0, height()-20, width(), 20 );
    label->setFrameStyle( QFrame::Box | QFrame::Raised );
    label->setLineWidth( 1 );
    label->setAlignment( AlignLeft );

    plot = new LogPlot(this);
    CHECK_PTR(plot);
    plot->setGeometry(0,menu->height(),width(),label->y()-menu->height());
    plot->SetParameters(CHART_LOOK_SCOPE,"f/Hz","L/dB","Spectrum Analyzer");
    plot->StartTimer(40);
    plot->AllocPoints(511);
    plot->SetLimits(20,20e3,-120,0);
    
    connect( this,  SIGNAL(explain(const char *)),
	     label, SLOT(setText(const char *)) );

    setMinimumSize( 100, 80 );
    isFreezed=FALSE;
}

void Gui::RecordOn()
{
    if(isFreezed)
    {
	isFreezed=FALSE;
	StartRecording();
    }
}

void Gui::ResetPoints()
{
    int i;
    for (i=0;i<511;i++)
      plot->points[i].y=-200;
}

void Gui::menu_quit()
{
    plot->StopTimer();
    Quit();
    qApp->quit();
}

void Gui::menu_export()
{
    int i;
    ofstream fs("spec.out");
    
    for (i=0;i<511;i++)
    {
	fs << plot->points[i].x << " " << plot->points[i].y << endl;
    }
    fs.close;
    label->setText( "Data Exported to file spec.out.");
}

void Gui::menu_nodelay()
{
    RecordOn();
    label->setText( "No delay." );
    plot->StartTimer(0);
}

void Gui::menu_delay10()
{
    RecordOn();
    label->setText( "Delay set to 10 ms." );
    plot->StartTimer(10);
}

void Gui::menu_delay20()
{
    RecordOn();
    label->setText( "Delay set to 20 ms.");
    plot->StartTimer(20);
}

void Gui::menu_delay40()
{
    RecordOn();
    label->setText( "Delay set to 40 ms." );
    plot->StartTimer(40);
}

void Gui::menu_delay80()
{
    RecordOn();
    label->setText( "Delay set to 80 ms." );
    plot->StartTimer(80);
}

void Gui::menu_delay160()
{
    RecordOn();
    label->setText( "Delay set to 160 ms." );
    plot->StartTimer(160);
}

void Gui::menu_delay320()
{
    RecordOn();
    label->setText( "Delay set to 320 ms." );
    plot->StartTimer(320);
}

void Gui::menu_freeze()
{    
    if(isFreezed)
    {
	label->setText( "Unfreezed." );
	plot->ContinueTimer();
	StartRecording();
    }
    else
    {	
	label->setText( "Freezed." );
	plot->StopTimer();
	StopRecording();
    }
    isFreezed = !isFreezed;
}

void Gui::menu_40000()
{    
    label->setText( "Sample rate set to 40000." );
    SetSF(40000);
    ResetPoints();
}

void Gui::menu_20000()
{    
    label->setText( "Sample rate set to 20000." );
    SetSF(20000);
    ResetPoints();
}

void Gui::menu_10000()
{    
    label->setText( "Sample rate set to 10000." );
    SetSF(10000);
    ResetPoints();
}

void Gui::menu_kaiser()
{
    label->setText( "Kaiser window function selected.");
    SetWinKaiser();
}

void Gui::menu_blackman()
{
    label->setText( "Blackman window function selected.");
    SetWinBlackman();
}

void Gui::menu_rectangle()
{
    label->setText( "Rectangular window function selected.");
    SetWinRectangle();
}

void Gui::menu_peak()
{
    label->setText( "Peak display selected.");
    ResetPoints();
    SetOpmode(1);
}

void Gui::menu_reset()
{
    label->setText( "Plot reseted.");
    ResetPoints();
}

void Gui::menu_normal()
{
    label->setText( "Normal display selected.");
    SetOpmode(0);
}

void Gui::menu_nop()
{
    label->setText( "Not Implemented yet." );
}

void Gui::resizeEvent( QResizeEvent * )
{
    label->setGeometry( 0, height()-20, width(), height() );
    plot->setGeometry(0,menu->height(),width(),label->y()-menu->height());
}
